# The Netlogon Project
#
# db.py - database access
#
# $Id: db.py,v 1.4 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
# Copyright (C) 1999 Kent Engstrm and Cendio Systems

import MySQL
import sys

# Allmn felkod
class DBError(Exception): pass

# Srskild felkod fr load_*_one-funktionerna
class DBNotSingleton(DBError): pass

dbh = None

#
# Gr en uppkoppling till databasen. Observera att handtaget
# lagras i en modulglobal variabel och inte behver skickas med
# vid senare anrop.
# Argumentet host ska vara "localhost" fr lokal uppkoppling.
#

def connect(host, user, password, database):
    global dbh
    dbh = MySQL.connect(host, user, password)
    dbh.selectdb(database)

# Utfr en SQL-frga.
# Denna funktion anvnds d svarsvrdet inte r av betydelse,
# t.ex. vid en INSERT eller UPDATE.

def query(query):
    try:
        return dbh.do(query)
        
    except MySQL.error, err_info:
        raise DBError, err_info

# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet ska
# returneras som en lista (raderna) av listor (kolumnerna).

def load_list_list(query):
    try:
        return dbh.do(query)
        
    except MySQL.error, err_info:
        raise DBError, err_info

# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet ska
# returneras som en lista (raderna) av tupler (kolumnerna).
# Anvnd inte denna funktion om load_list_list duger.

def load_tuple_list(query):
    try:
        ret = dbh.do(query)
        return map(lambda x: tuple(x), ret)
        
    except MySQL.error, err_info:
        raise DBError, err_info

# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet r en enda kolumn och
# svaret ska vara en lista av vrden (inte en lista av listor med
# ett element i varje dellista).

def load_one_column(query):
    try:
        ret = dbh.do(query)
        return map(lambda x: x[0], ret)
    except MySQL.error, err_info:
        raise DBError, err_info

# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet bara ska omfatta en rad och svaret ska
# returneras som en lista av kolumnerna. Det srskilda felet DBNotSingleton
# ges om inte resultatet r exakt en rad.

def load_list_one(query):
    try:
        res = dbh.do(query)
        try:
            [singleton] = res
            return singleton
        except:
            raise DBNotSingleton
    except MySQL.error, err_info:
        raise DBError, err_info


# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet ska
# returneras som en lista (raderna) av dictionaries (kolumnerna).
# Nyckel i dictionaryt r normalt "tabellnamn.fltnamn".

# Anvnds bara denna funktion om anropet till load_list_list skulle
# bli fr omstndligt att hantera.

def load_dict_list(query):
    try:
        sth = dbh.query(query)
        return sth.fetchdict()
    except MySQL.error, err_info:
        raise DBError, err_info

# Utfr en SQL-frga.
# Denna funktion anvnds d resultatet bara ska omfatta en rad och
# returneras som ett  dictionaries ver kolumnerna.
# Nyckel i dictionaryt r normalt "tabellnamn.fltnamn".
# Det srskilda felet DBNotSingleton ges om inte resultatet r exakt en rad.

# Anvnds bara denna funktion om anropet till load_list_one skulle
# bli fr omstndligt att hantera.

def load_dict_one(query):
    try:
        sth = dbh.query(query)
        dictlist = sth.fetchdict()
        try:
            [singleton] = dictlist
            return singleton
        except:
            raise DBNotSingleton
    except MySQL.error, err_info:
        raise DBError, err_info

# Skydda strngen s att den kan lggas mellan enkelfnuttar (')
# i SQL utan att stngen avslutas fr tidigt.

def escape(str):
    return MySQL.escape(str)



