/* The Netlogon Project

   delfw.c - delete firewall rules (thus disallowing access for a single IP address)
   
   $Id: delfw.c,v 1.5 2001/03/08 14:54:51 kent Exp $
   
   Copyright (C) 2000,2001 UNIT, Linkping University
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define TIMES 4

main(int argc, char **argv)
{
  int i;
  char buf[256];
  int rc = 0;
  char *ip;
  struct in_addr ip_struct;
  int masq = 0;

  /* Argument count */

  if (argc == 2)
    {
      ip = argv[1];
    }
  else if (argc == 3)
    {
      ip = argv[2];
      if (!strcmp(argv[1],"-m"))
	{
	  masq = 1;
	}
    }
  else
    {
      exit(1);
    }

  /* Argument length and syntax*/
  if (strlen(ip) > 32)
    {
      exit(1);
    }

  if (!inet_aton(ip, &ip_struct))
    {
      exit(1);
    }

  /* Step 1: Delete from fw-users chain. Repeat TIMES times, to get rid of
     duplicate rules. Defensive, but very useful during testing :-)*/
  for (i=0; i<TIMES; i++)
    {
      if (masq)
	{
	  sprintf(buf, "/sbin/ipchains -D fw-users -s %s -j MASQ >/dev/null 2>&1", ip);
	}
      else
	{
	  sprintf(buf, "/sbin/ipchains -D fw-users -b -s %s -j ACCEPT >/dev/null 2>&1", ip);
	}
      if (system(buf) <0)
	{
	  rc = 1;
	  break;
	}
    }

  /* Step 2: Delete from in-users chain. Repeat TIMES times, to get rid of
     duplicate rules. Defensive, but very useful during testing :-)*/
  for (i=0; i<TIMES; i++)
    {
      sprintf(buf, "/sbin/ipchains -D in-users -s %s -j ACCEPT >/dev/null 2>&1", ip);
      if (system(buf) <0)
	{
	  rc = 1;
	  break;
	}
    }

  exit(rc);
}
