/* The Netlogon Project

   ledd.c - LED daemon
   
   $Id: ledd.c,v 1.3 2001/03/08 14:54:51 kent Exp $
   
   Copyright (C) 2000,2001 UNIT, Linkping University
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>
#include <asm/io.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>


#define BASE 0x278
#define PAUSE (50*1000)

#define ERROR_CYCLES 100
#define NOTICE_CYCLES 100

/* Low-level LED functions */

void init_leds(void)
{
  if (ioperm(BASE, 3, 1) == -1)
    {
      perror("ioperm");
      exit(1);
    }
}

/* numbered controls the numbered LEDS:

   0x01 light LED no 0
   0x02 light LED no 1
...
   0xf0 light LED no 7
...
   0xf3 light LED no 0, 1 and 7

   ok and alert controls the "OK" and "ALERT" leds */

void led(int numbered, int ok, int alert)
{
  int ctrl = 0;

  if (ok) ctrl |= 1;
  if (!alert) ctrl |= 4;

  outb(numbered & 0xff, BASE);
  outb(ctrl, BASE + 2);
}

void test_leds(void)
{
  /* LED self-test */

  led(0xff, 1, 1);
  usleep(500*1000);

  led(0, 0, 0);
  usleep(500*1000);

}

/* /proc/net/dev interface for ethN stats
   We are interested in the received and transmitted packets
*/

#define INTERFACES 4
int rx_packets_old[INTERFACES];
int tx_packets_old[INTERFACES];

/* Return what to show on the numbered LED:s */
int check_proc_net_dev(void)
{
  FILE *f;
  char line[256];
  int conv;
  int ifno;

  char ifname[256];
  int rx_bytes;
  int rx_packets;
  int rx_errs;
  int rx_drop;
  int rx_fifo;
  int rx_frame;
  int rx_compressed;
  int rx_multicast;
  int tx_bytes;
  int tx_packets;
  int tx_errs;
  int tx_drop;
  int tx_fifo;
  int tx_colls;
  int tx_carrier;
  int tx_compressed;

  int rx_led, tx_led;
  int numbered = 0;

  if ((f = fopen("/proc/net/dev", "r")) == NULL)
    return;

  while (fgets(line, 256, f) != NULL)
    {
      conv = sscanf(line, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
		    ifname,
		    &rx_bytes,
		    &rx_packets,
		    &rx_errs,
		    &rx_drop,
		    &rx_fifo,
		    &rx_frame,
		    &rx_compressed,
		    &rx_multicast,
		    &tx_bytes,
		    &tx_packets,
		    &tx_errs,
		    &tx_drop,
		    &tx_fifo,
		    &tx_colls,
		    &tx_carrier,
		    &tx_compressed);
      if (conv != 17) continue;
      if (strstr(ifname, "eth") != ifname) continue;
      ifno = atoi(ifname + 3);
      if ((ifno < 0) || (ifno >= INTERFACES)) continue;
      
      if (rx_packets != rx_packets_old[ifno])
	rx_led = 1;
      else
	rx_led = 0;
      rx_packets_old[ifno] = rx_packets;

      if (tx_packets != tx_packets_old[ifno])
	tx_led = 1;
      else
	tx_led = 0;
      tx_packets_old[ifno] = tx_packets;

      numbered |= (rx_led + 2 * tx_led) << (2 * ifno);
    }
  fclose(f);
  return numbered;
}

/* Socket interface */

#define SOCKNAME "/opt/netlogon/sock/ledd"
#define CMD_NOTICE 1
#define CMD_ERROR 2

int s;
void init_socket(void)
{
   struct sockaddr_un  addr;
   int addrlen;

   if ((s = socket(AF_UNIX,SOCK_DGRAM,0)) < 0)
     return;

   unlink(SOCKNAME);

   bzero((char *) &addr, sizeof(addr));
   addr.sun_family = AF_UNIX;
   strcpy(addr.sun_path, SOCKNAME);
   addrlen=strlen(addr.sun_path) + sizeof(addr.sun_family);
   if(bind(s,(struct sockaddr *)&addr,addrlen)<0)
     s=-1;
}

int watch_socket(int sec, int usec, int *notice, int *error)
{
  fd_set infds;
  struct timeval tv;
  int selret;
  struct sockaddr_un  addr;
  int addrlen;
  char cmdbuf[2];
  int n;

  *notice = 0;
  *error = 0;

  if (s<0)
    {
      usleep(sec * 1000000 + usec);
      return;
    }
  
  FD_ZERO(&infds);
  FD_SET(s, &infds);
  tv.tv_sec=sec;
  tv.tv_usec=usec;
  selret = select(s + 1, &infds, NULL, NULL, &tv);
     
  if (FD_ISSET(s, &infds))
    {
      addrlen = sizeof(addr);
      n = recvfrom(s, cmdbuf, 2, 0, (struct sockaddr *)&addr,&addrlen);
      if (n == 2)
	{
	  switch (cmdbuf[0])
	    {
	    case CMD_NOTICE:
	      *notice = cmdbuf[1];
	      break;
	    case CMD_ERROR:
	      *error = cmdbuf[1];
	      break;
	    }
	}
    }
}

/* MAIN */

int main()
{
  int numbered;
  int ok_time=0;

  int error=0;   /* The error we are displaying */
  int new_error;   /* The error we receive */
  int error_time=0;  /* Number of cycles left for showing this error */

  int notice=0;  /* The notice we are displaying */
  int new_notice;  /* The notice we receive */
  int notice_time=0;  /* Number of cycles left for showing this notice */

  init_socket();
  init_leds();
  test_leds();

  /* main loop */
  
  while(1)
    {
      if (error)
	{
	  led(error, 0, error_time & 0x04);
	}
      else if (notice)
	{
	  led(notice, notice_time & 0x04, 0);
	}
      else
	{
	  numbered = check_proc_net_dev();
	  led(numbered, ok_time >= 10, 0);
	}

      if (error_time)
	{
	  error_time--;
	  if (error_time == 0)
	    error = 0;
	}
	
      if (notice_time)
	{
	  notice_time--;
	  if (notice_time == 0)
	    notice = 0;
	}
	
      watch_socket(0, PAUSE, &new_notice, &new_error);
      if (new_error) {
	error = new_error;
	error_time = ERROR_CYCLES;
      }
      if (new_notice) {
	notice = new_notice;
	notice_time = NOTICE_CYCLES;
      }
      ok_time = (ok_time+1) % 100;
    }
}
