#!/usr/bin/python

# The Netlogon Project
#
# logon.cgi - logon CGI handler
#
# $Id: logon.cgi,v 1.10 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import path

import cgi
import sys
import os
import string
import traceback

import template
import auth      # authorization
import network   # low level stuff
import conf      # configuration file reading
import ipaddr    # classes for addresses etc.
import conndb    # connection database
import language  # i18n
import log       # logging (via syslog)

#
# Display error message in correct language
#
def show_error(code, tags = {}):
    temp = template.LangTemplate("error", lang)
    txt = template.substitute_tags_string(language.get(code, lang),
                                          tags)
    print temp.instantiate({"txt": txt})
    return

#
# Validate the IP address
# If the address is not valid: show complaint and exit
# If the address is valid:
#	store it in ip_address
#	get the connection object to connection

def validate_ip_address():
    global ip_address
    global connection
    
    # Get address of browser
    try:
        ip_address_string = os.environ["REMOTE_ADDR"]
        ip_address = ipaddr.ipaddr(ip_address_string)
    except:
        ip_address_string = "UNKNOWN"
        ip_address = None

    # Check address against interface config
    if ip_address is None:
        log.log("unknown address connecting to web server", log.LOG_WARNING, led_error = 32)
        show_error("bad_address", {"ip": ip_address.ip_str()})
        sys.exit(0)

    # Read interfaces from config file
    interfaces = conf.Interfaces()

    # Get connection object
    connection = conndb.get_connection(ip_address)

    # Check address against interface config
    if not interfaces.acceptable_client_address(ip_address):
        connection.log_failed_logging_on(cause = "bad-address")
        show_error("bad_address", {"ip": ip_address.ip_str()})
        sys.exit(0)

#
# FUNCTION: Entry screen
#   - If the user is not logged in, show logon screen
#   - If the user is logged in, show logoff screen

def function_entry():
    if connection.is_on():
        function_show_logoff()
    else:
        function_show_logon()

#
# FUNCTION: Show a logon screen
#

def function_show_logon():
    # Change state to LOGGING_ON
    try:
        connection.goto_logging_on(ip_address)
    except conndb.ConnDBNoArpInfo:
        connection.log_failed_logging_on(cause="no-arp")
        return show_error("bad_ether")
    except:
        connection.log_failed_logging_on(cause="internal")
        return show_error("bad_show_logon")

    # Get authentication alternatives
    a = auth.Auth(lang)
    auth_list = a.get_codes_and_names()
    auth_str = string.join(map(lambda x: "<OPTION value=%s>%s" % (x[0],x[1]), auth_list), "\n")

    # Show logon page
    temp = template.LangTemplate("logon", lang)
    print temp.instantiate({"lang": lang,
                            "auth_options": auth_str,
                            "ip": ip_address.ip_str(),
                            "ether": connection.get_ether()}
                           )

#
# FUNCTION: Handle logon
#

def function_logon():
    # Check that the state is right (CS_LOGGING_ON)
    if not connection.is_logging_on():
        return show_error("not_logging_on")

    # Handle logon data (type, user, password)
    # Note: no field may be blank. Change this code if a blank field is OK.
    try:
        logon_type     = cgiform["type"].value[:32]
        logon_user     = cgiform["user"].value[:32]
        logon_password = cgiform["password"].value[:32]
    except:
        return show_error("logon_data_missing")

    # Check with auth module
    a = auth.Auth(lang)
    try:
        a.authenticate(logon_type, logon_user, logon_password)
    except auth.AuthBadLogin:
        connection.log_failed_on(logon_type, logon_user, cause="bad-auth")
        return show_error("logon_failed_user")
    except:
        connection.log_failed_on(logon_type, logon_user, cause="internal")
        return show_error("logon_failed_system")

    # Change state to ON
    try:
        connection.goto_on(logon_type, logon_user)
    except:
        return show_error("bad_logon")

    # Show success page
    url = "http://%s/" % gc.get_default("www","servername","SERVERNAME-MISSING")
    temp = template.LangTemplate("logged_on", lang)
    print temp.instantiate({"lang": lang,
                            "ip": ip_address.ip_str(),
                            "ether": connection.get_ether(),
                            "url": url}
                           )

#
# FUNCTION: Show a logoff screen
#

def function_show_logoff():
    # Show logoff page
    temp = template.LangTemplate("logoff", lang)
    print temp.instantiate({"lang": lang,
                            "ip": ip_address.ip_str(),
                            "ether": connection.get_ether()}
                           )

#
# FUNCTION: Handle logoff
#

def function_logoff():
    # Check that the state is right (CS_ON)
    if not connection.is_on():
        connection.log_failed_off(cause="not-on")
        return show_error("not_on")

    # Change state to OFF
    try:
        connection.goto_off(cause="user")
    except:
        connection.log_failed_off(cause="internal")
        return show_error("bad_logoff")

    # Show success page
    temp = template.LangTemplate("logged_off", lang)
    print temp.instantiate({"lang": lang})

#
# MAIN
#

# Logging

log.init("logon")

# Get access to general.conf
gc = conf.Config("general")

# Head

print template.cgi_head()
cgiform = cgi.FieldStorage()

# Until we know more, assume swedish
lang = "sv"

# Validate the IP address (exit this program if its bad)
# This also created the connection object
validate_ip_address()

# Override lang?
if cgiform.has_key("lang"):
    try:
        maybe_lang = cgiform["lang"].value
        if maybe_lang in ["sv", "en"]:
            connection.set_lang(maybe_lang)
    except:
        pass
lang = connection.get_lang()


# Dispatch on "func"
try:
    func = cgiform["func"].value
except:
    func = "entry"
    
dispatch_table = {
    "entry": function_entry,
    "logon": function_logon,
    "logoff": function_logoff,
    }

# Yes, this is a function call. Orthogonality rules...
try:
    dispatch_table[func]()
except:
    (type, value, tb) = sys.exc_info()
    log.log("internal error type=%s info=%s backtrace=[%s]" % \
            (type,
             value,
             string.join(traceback.format_list(traceback.extract_tb(tb)),"   ")),
            log.LOG_ALERT)
    show_error("internal_error")
    
