# The Netlogon Project
#
# network.py - low-level network handling
#
# $Id:#
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import path
import os
import string
import re
import log

class NetworkError: pass # General error

# Get the interface name and ethernet address from the ARP table
def get_ifname_and_ether(ip):
    ifname = ether = None
    ip_string = ip.ip_str()
    try:
        f = open("/proc/net/arp")
        for line in f.readlines():
            fields = string.split(line)
            import sys
            if fields[0] == ip_string:
                ether = fields[3]
                ifname = fields[5]
                break
        f.close()
        return (ifname, ether)
    except:
        return None

# Fix the ethernet adress of a given IP address
def fix_ether(ip, ether):
    rc = os.system("%s/fixmac %s %s" % (path.bindir, ip.ip_str(), ether))
    if rc <> 0:
        log.log("failure fixing ip %s to ether %s" % (ip.ip_str(), ether),
                led_error = 2)
        raise NetworkError
    
# Unfix the ethernet adress of a given IP address
def unfix_ether(ip):
    rc = os.system("%s/unfixmac %s" % (path.bindir, ip.ip_str()))
    if rc <> 0:
        log.log("failure unfixing ip %s from some ether" % (ip.ip_str()),
                led_error = 3)
        raise NetworkError
    
# Add a forwarding rule allowing the IP to communicate
def add_forwarding_rule(ip, masquerade):
    if masquerade:
        mflag="-m "
    else:
        mflag=""
        
    rc = os.system("%s/addfw %s%s" % (path.bindir, mflag, ip.ip_str()))
    if rc <> 0:
        log.log("failure adding forwarding rule for %s" % (ip.ip_str()),
                led_error = 4)
        raise NetworkError
    
# Delete a forwarding rule 
def del_forwarding_rule(ip, masquerade):
    if masquerade:
        mflag="-m "
    else:
        mflag=""
        
    rc = os.system("%s/delfw %s%s" % (path.bindir, mflag, ip.ip_str()))
    if rc <> 0:
        log.log("failure deleting forwarding rule for %s" % (ip.ip_str()),
                led_error = 5)
        raise NetworkError
    
# Do an arp ping of an IP address on a certain interface.
# Return ethernet address or None if no response.

re_good_arp = re.compile("Unicast reply from [0-9.]* \[([0-9A-F:]*)\]")
def arp_ping(ip, ifname, count = 1):
    data = ""
    try:
        f = os.popen("/sbin/arping -c %d -I %s %s 2>&1" % (count, ifname, ip.ip_str()))
        data = f.read()
        rc = f.close()
        if rc is not None: return None
    except:
        return None

    # Check data
    m = re_good_arp.search(data)
    if m:
        return m.group(1)
    else:
        return None

# Get forwarding statistics for a certain IP
# Return value is None on failure, else a tuple of
# (from_packets, from_bytes, to_packets, to_bytes)

def get_forwarding_stats(ip):
    try:
        ip_str = ip.ip_str()
        f =  os.popen("%s/listfw" % (path.bindir))
        data = f.read()
        f.close()
    except:
        return (0,0,0,0)

    re_from_ip = re.compile("^ *([0-9]+) +([0-9]+) .* %s +0.0.0.0/0" % ip_str, re.M)
    m_from = re_from_ip.search(data)

    re_to_ip = re.compile("^ *([0-9]+) +([0-9]+) .* 0.0.0.0/0 +%s " % ip_str, re.M)
    m_to = re_to_ip.search(data)

    if m_from is None or m_to is None:
        return (0,0,0,0)

    return (string.atoi(m_from.group(1)),
            string.atoi(m_from.group(2)),
            string.atoi(m_to.group(1)),
            string.atoi(m_to.group(2)))

# Collect statistics for all interfaces
# Returns a list (all interfaces) of lists (17 elements as shown below):
# All integers are converted to Python longs

#Inter-|   Receive                                                |  Transmit
# face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed

def collect_interface_statistics():
    ret = []
    f = open("/proc/net/dev")
    for line in f.readlines():
        try:
            raw = filter(lambda x: x<>"", re.split("[: \n]+", line))
            if len(raw) == 17:
                row = [raw[0]] + map(string.atol, raw[1:])
                ret.append(row)
        except:
            pass
    f.close()
    return ret
