# The Netlogon Project
#
# switch_snmp.py - gather data from switches using SNMP
#
# $Id: switch_snmp.py,v 1.2 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import simple_snmp
import string

# OID:s
OID_mib_2 =                ".1.3.6.1.2.1"

# Relevant OID:s from IF-MIB
OID_interfaces =           OID_mib_2 + ".2"
OID_ifTable =              OID_interfaces + ".2"
OID_ifIndex =              OID_ifTable + ".1.1"
OID_ifDescr =              OID_ifTable + ".1.2"

OID_ifMIB =                OID_mib_2 + ".31"
OID_ifMIBObjects =         OID_ifMIB + ".1"
OID_ifXTable =             OID_ifMIBObjects + ".1"
OID_ifAlias =              OID_ifXTable + ".1.18"

# Relevant OID:s from BRIDGE-MIB
OID_dot1dBridge =          OID_mib_2 + ".17"

OID_dot1dBase =            OID_dot1dBridge + ".1"
OID_dot1dBasePortTable =   OID_dot1dBase + ".4"
OID_dot1dBasePort =        OID_dot1dBasePortTable + ".1.1"
OID_dot1dBasePortIfIndex = OID_dot1dBasePortTable + ".1.2"

OID_dot1dTp =              OID_dot1dBridge + ".4"
OID_dot1dTpFdbTable =      OID_dot1dTp + ".3"
OID_dot1dTpFdbAddress =    OID_dot1dTpFdbTable + ".1.1"
OID_dot1dTpFdbPort =       OID_dot1dTpFdbTable + ".1.2"
OID_dot1dTpFdbStatus =     OID_dot1dTpFdbTable + ".1.3"

# Return interface name and description/alias given switch and ether
# Return (None, None) on error or if information cannot be found
# Convert spaces to "_"!

def get_port_data_from_ether(switch_ip, community, ether):
    ipstring = switch_ip.ip_str()

    s = simple_snmp.Session(ipstring, community)

    # First, we use dot1dBasePortTable to map ethernet address to
    # port number
    try:
        port = int(s[OID_dot1dTpFdbPort + "." + \
                     simple_snmp.ether_to_oid_string(ether)])
    except:
        return (None, None)

    # Now, map port to ifindex using dot1dBasePortTable
    try:
        ifIndex = int(s[OID_dot1dBasePortIfIndex + (".%d" % port)])
    except:
        return (None, None)

    # Now, get the ifDescr (interface name) using ifTable
    try:
        ifDescr = s[OID_ifDescr + (".%d" % ifIndex)]
    except:
        ifDescr = "ifIndex %d" % ifIndex

    
    # Now, get the ifAlias (name specified by manager) using ifXTable
    try:
        ifAlias = s[OID_ifAlias + (".%d" % ifIndex)]
    except:
        ifAlias = ifDescr

    return (string.replace(ifDescr, " ", "_"),
            string.replace(ifAlias, " ", "_"))
