# The Netlogon Project
#
# template.py - template substitution for strings / files / web pages
#
# $Id: template.py,v 1.4 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
# Copyright (C) 1999 Kent Engstrm, Cendio Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import string
import os

import path

# Hjlpfunktion: ppna frsta filen som passar, fr lsning
# Ge None om ingen fil kan ppnas

def multi_open(filelist):
    for filename in filelist:
	try:
	    return open(filename)
	except:
	    pass
    return None

def substitute_tags_string(str, tags):
    """Perform tag substitution on STR with the given TAGS.

    Arguments:
      STR     A string which may contain tags on the form #KEYWORD#.
      TAGS    A dictionary with tags (strings) as keys.  The value
              should also be a string.  Hash-marks will be prepended
	      and appended to each key before it is used to search for
	      in the STR argument.

    This returns the string with all substitutions performed.
    """

    for tag in tags.keys():
        str = string.replace(str, "#" + tag + "#", tags[tag])

    return str


def cgi_head(content_type = "text/html"):
    """Return the top lines needed in a CGI response."""
    
    return "Content-Type: %s\n\n" % content_type
    
#
# OBJEKTORIENTERAT GRNSSNITT

class StringTemplate:
    def __init__(self, template_string):
        self._template = template_string

    def instantiate(self, tags = {}):
        return substitute_tags_string(self._template, tags)
    
class FileTemplate(StringTemplate):
    def __init__(self, template_file):
        if type(template_file) == type(""):
            infile = open(template_file, 'r')
        else:
            infile = multi_open(template_file)

        self._template = infile.read()
        infile.close()

class LangTemplate(FileTemplate):
    def __init__(self, name, lang):
        self._filename = os.path.join(path.templatedir,
                                      name + "_" + lang + ".html")
        infile = open(self._filename, 'r')
        self._template = infile.read()
        infile.close()
    
